import{aN as m,aO as d,aF as b,aP as h,aQ as o}from"./index-DS5wmJTE.js";import{c as p,d as g}from"./_baseUniq-CzH_Y1Oj.js";function v(a,s){var e=-1,i=m(a)?Array(a.length):[];return p(a,function(r,t,n){i[++e]=s(r,t,n)}),i}function E(a,s){var e=d(a)?b:v;return e(a,g(s))}function x(a,s){return a<s}function y(a,s,e){for(var i=-1,r=a.length;++i<r;){var t=a[i],n=s(t);if(n!=null&&(f===void 0?n===n&&!h(n):e(n,f)))var f=n,u=t}return u}function L(a){return a&&a.length?y(a,o,x):void 0}export{x as a,y as b,v as c,L as d,E as m};
